<?php

/**
 * PannelController : Controller du pannel
 *
 * web-DPO : Outil de gestion de vos traitements dans le cadre de la 
 * réglementation relative à la protection des données personnelles (RGPD)
 * 
 * Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 *
 * Licensed under the GNU Affero General Public License version 3 License - AGPL v3
 * For full copyright and license information, please see the "LICENSE" file.
 * Redistributions of files must retain the above copyright notice.
 * 
 * @copyright   Copyright (c) Libriciel SCOP (https://www.libriciel.fr/)
 * @link        https://www.libriciel.fr/web-dpo/
 * @since       web-DPO v1.0.0
 * @license     [GNU Affero General Public License version 3](http://www.gnu.org/licenses/agpl-3.0.html) - AGPL v3
 * @version     v1.0.0
 * @package     App.Controller
 */

App::uses('EtatFiche', 'Model');
App::uses('ListeDroit', 'Model');

class PannelController extends AppController {

    public $uses = [
        'Pannel',
        'Fiche',
        'Users',
        'OrganisationUser',
        'Droit',
        'EtatFiche',
        'Commentaire',
        'Modification',
        'Notification',
        'Historique',
        'Organisation',
        'Valeur',
        'Norme'
    ];
    public $components = [
//        'FormGenerator.FormGen',
        'Droits',
        'Banettes'
    ];
    public $helpers = [
        'Banettes'
    ];

    /**
     * Vérification de l'accès aux actions en fonction du profil de l'utilisateur connecté.
     * Les vérifications de l'accès aux enregistrements se font dans les méthodes d'actions.
     */
    public function beforeFilter() {
        parent::beforeFilter();

        $action = Inflector::underscore($this->request->params['action']);
        $anyone = ['drop_notif', 'index', 'technique', 'valid_notif'];
        $listeDroitIndex = [ListeDroit::REDIGER_TRAITEMENT, ListeDroit::VALIDER_TRAITEMENT, ListeDroit::VISER_TRAITEMENT];

        if ($action === 'all_traitements') {
            $this->Droits->assertAuthorized([ListeDroit::CONSULTER_ALL_TRAITEMENT]);
            $this->Droits->assertNotSu();
        } elseif ($action === 'archives') {
            $this->Droits->assertAuthorized([ListeDroit::REDIGER_TRAITEMENT]);
            $this->Droits->assertNotSu();
        } elseif ($action === 'attente') {
            $this->Droits->assertAuthorized([ListeDroit::REDIGER_TRAITEMENT]);
            $this->Droits->assertNotSu();
        } elseif ($action === 'consulte') {
            $this->Droits->assertAuthorized([ListeDroit::VALIDER_TRAITEMENT, ListeDroit::VISER_TRAITEMENT]);
            $this->Droits->assertNotSu();
        } elseif ($action === 'encours_redaction') {
            $this->Droits->assertAuthorized([ListeDroit::REDIGER_TRAITEMENT]);
            $this->Droits->assertNotSu();
        } elseif ($action === 'get_historique') {
            $this->Droits->assertNotSu();
        } elseif ($action === 'initialisation') {
            $this->Droits->assertDpo();
            $this->Droits->assertNotSu();
        } elseif ($action === 'notif_afficher') {
            $this->Droits->assertNotSu();
        } elseif ($action === 'parcours') {
            $this->Droits->assertNotSu();
        } elseif ($action === 'recu_consultation') {
            $this->Droits->assertAuthorized([ListeDroit::VISER_TRAITEMENT]);
            $this->Droits->assertNotSu();
        } elseif ($action === 'recu_validation') {
            $this->Droits->assertAuthorized([ListeDroit::VALIDER_TRAITEMENT]);
            $this->Droits->assertNotSu();
        } elseif ($action === 'refuser') {
            $this->Droits->assertAuthorized([ListeDroit::REDIGER_TRAITEMENT]);
            $this->Droits->assertNotSu();
        } elseif ($action === 'supprimer_la_notif') {
            $this->Droits->assertNotSu();
        } elseif ($action === 'superadmin') {
            $this->Droits->assertSu();
        } elseif (in_array($action, $anyone) === false) {
            throw new ForbiddenException(__d('default', 'default.flasherrorPasDroitPage'));
        }
    }
    
    public function superadmin() {
        // @todo : issues/255
//        if (!empty($this->Session->read('Organisation'))) {
//            $this->Session->delete('Organisation');
//        }

        $this->set('title', __d('default', 'default.titreAdministrationApplication'));
    }

    public function technique() {
        if (true === $this->Droits->authorized([ListeDroit::REDIGER_TRAITEMENT, ListeDroit::VALIDER_TRAITEMENT, ListeDroit::VISER_TRAITEMENT])) {
            $this->redirect(['action' => 'index']);
        }

        $this->set('title', __d('default', 'default.titreAdministrationDeEntite'));
    }
    
    /**
     * Accueil de la page, listing des fiches et de leurs catégories
     * 
     * @access public
     * @created 02/12/2015
     * @version V1.0.0
     */
    public function index() {
        $this->Session->delete('Auth.User.uuid');

        if ($this->Droits->isSu() === true) {
            return $this->superadmin();
        }

        if (true !== $this->Droits->authorized([ListeDroit::REDIGER_TRAITEMENT, ListeDroit::VALIDER_TRAITEMENT, ListeDroit::VISER_TRAITEMENT])) {
            $this->redirect(['action' => 'technique']);
        }
        
        $banettes = [];
        $limit = 5;
        
        $this->set('title', __d('pannel', 'pannel.titreTraitement'));
        
        $return = $this->_listValidants();
        $nbUserValideur = count($return['validants']);
        $nbUserConsultant = count($return['consultants']);
        
        $enumsNormes = $this->_enumsNorme();
        
        $query = [
            'conditions' => [
                'abroger' => false
            ],
            'fields' => ['id', 'norme', 'numero', 'libelle', 'description'],
            'order' => ['norme', 'numero']
        ];
        $normes = $this->Norme->find('all', $query);
        
        $options_normes = Hash::combine(
            $normes,
            '{n}.Norme.id',
            [
                '%s-%s : %s',
                '{n}.Norme.norme',
                '{n}.Norme.numero',
                '{n}.Norme.libelle'
            ]
        );
        $this->set(compact('options_normes'));
              
        if ($this->Droits->authorized(ListeDroit::REDIGER_TRAITEMENT)) {
            // En cours de rédaction
            $query = $this->Banettes->queryEnCoursRedaction();
            $banettes['encours_redaction'] = [
                'results' => $this->Fiche->find('all', $query + ['limit' => $limit]),
                'count' => $this->Fiche->find('count', $query),
                'nbUserValideur' => $nbUserValideur,
                'nbUserConsultant' => $nbUserConsultant,
                'enumsNormes' => $enumsNormes
            ];

            // En attente
            $query = $this->Banettes->queryAttente();
            $banettes['attente'] = [
                'results' => $this->Fiche->find('all', $query + ['limit' => $limit]),
                'count' => $this->Fiche->find('count', $query),
                'nbUserValideur' => $nbUserValideur,
                'enumsNormes' => $enumsNormes
            ];
            
            // Traitement refusés
            $query = $this->Banettes->queryRefuser();
            $banettes['refuser'] = [
                'results' => $this->Fiche->find('all', $query + ['limit' => $limit]),
                'count' => $this->Fiche->find('count', $query),
                'enumsNormes' => $enumsNormes
            ];
            
            // Mes traitements validés et insérés au registre
//            $query = $this->Banettes->queryArchives();
//            $banettes['archives'] = [
//                'results' => $this->Fiche->find('all', $query + ['limit' => $limit]),
//                'count' => $this->Fiche->find('count', $query)
//            ];
        }

        // Etat des traitements passés en ma possession
//        if ($this->Droits->authorized([ListeDroit::VALIDER_TRAITEMENT, ListeDroit::VISER_TRAITEMENT])) {
//            $query = $this->Banettes->queryConsulte();
//            $banettes['consulte'] = [
//                'results' => $this->Fiche->find('all', $query + ['limit' => $limit]),
//                'count' => $this->Fiche->find('count', $query),
//                'nbUserValideur' => $nbUserValideur
//            ];
//        }
        
        // Traitement reçu pour validation
        if ($this->Droits->authorized(ListeDroit::VALIDER_TRAITEMENT)) {
            $query = $this->Banettes->queryRecuValidation();
            $banettes['recuValidation'] = [
                'results' => $this->Fiche->find('all', $query + ['limit' => $limit]),
                'count' => $this->Fiche->find('count', $query),
                'nbUserValideur' => $nbUserValideur,
                'nbUserConsultant' => $nbUserConsultant,
                'enumsNormes' => $enumsNormes
            ];
        }
        
        // Traitement reçu pour consultation
        if ($this->Droits->authorized(ListeDroit::VISER_TRAITEMENT)) {
            $query = $this->Banettes->queryRecuConsultation();
            $banettes['recuConsultation'] = [
                'results' => $this->Fiche->find('all', $query + ['limit' => $limit]),
                'count' => $this->Fiche->find('count', $query),
                'enumsNormes' => $enumsNormes
            ];
        }
        
        $this->set(compact('banettes'));

        $notifications = $this->Notification->find('all', [
            'conditions' => [
                'Notification.user_id' => $this->Auth->user('id'),
                'Notification.vu' => false,
                'Notification.afficher' => false
            ],
            'contain' => [
                'Fiche' => [
                    'Valeur' => [
                        'conditions' => [
                            'champ_name' => 'outilnom'
                        ],
                        'fields' => [
                            'champ_name', 
                            'valeur'
                        ]
                    ]
                ]
            ],
            'order' => [
                'Notification.content'
            ]
        ]);
        $this->set('notifications', $notifications);

        $nameOrganisation = [];

        foreach ($notifications as $key => $value) {
            $nameOrganisation[$key] = $this->Organisation->find('first', [
                'conditions' => ['id' => $value['Fiche']['organisation_id']],
                'fields' => ['raisonsociale']
            ]);
        }
        $this->set('nameOrganisation', $nameOrganisation);
        
        $this->set('validants', $return['validants']);
        $this->set('consultants', $return['consultants']);
    }

    /**
     * Fonction qui récupère tous les traitements en cours de rédaction
     * 
     * @access public
     * @created 13/02/2017
     * @version V1.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    public function encours_redaction() {
        $this->set('title', __d('pannel', 'pannel.titreTraitementEnCoursRedaction'));
        
        $return = $this->_listValidants();
        $nbUserValideur = count($return['validants']);
        $nbUserConsultant = count($return['consultants']);
        
        // En cours de rédaction
        $query = $this->Banettes->queryEnCoursRedaction();
        $banettes['encours_redaction'] = [
            'results' => $this->Fiche->find('all', $query + ['limit' => 0]),
            'count' => $this->Fiche->find('count', $query),
            'nbUserValideur' => $nbUserValideur,
            'nbUserConsultant' => $nbUserConsultant,
            'enumsNormes' => $this->_enumsNorme()
        ];
        $this->set('banettes', $banettes);

        $this->set('validants', $return['validants']);
        $this->set('consultants', $return['consultants']);
    }

    /**
     * Fonction qui récupère tous les traitements en attente
     * 
     * @access public
     * @created 13/02/2017
     * @version V1.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    public function attente() {
        $this->set('title', __d('pannel', 'pannel.titreTraitementEnAttente'));

        $return = $this->_listValidants();
        $nbUserValideur = count($return['validants']);
        
        // En attente
        $query = $this->Banettes->queryAttente();
        $banettes['attente'] = [
            'results' => $this->Fiche->find('all', $query + ['limit' => 0]),
            'count' => $this->Fiche->find('count', $query),
            'nbUserValideur' => $nbUserValideur,
            'enumsNormes' => $this->_enumsNorme()
        ];
        
        $this->set('banettes', $banettes);
        $this->set('validants', $return['validants']);
    }

    /**
     * Fonction qui récupère tous les traitements refusés
     * 
     * @access public
     * @created 13/02/2017
     * @version V1.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    public function refuser() {
        $this->set('title', __d('pannel', 'pannel.titreTraitementRefuser'));

        // Traitement refusés
        $query = $this->Banettes->queryRefuser();
        $banettes['refuser'] = [
            'results' => $this->Fiche->find('all', $query + ['limit' => 0]),
            'count' => $this->Fiche->find('count', $query),
            'enumsNormes' => $this->_enumsNorme()
        ];
        $this->set('banettes', $banettes);
    }

    /**
     * Fonction qui récupère tous les traitements reçus pour validation
     * 
     * @access public
     * @created 13/02/2017
     * @version V1.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    public function recuValidation() {
        $this->Session->delete('Auth.User.uuid');

        $this->set('title', __d('pannel', 'pannel.titreTraitementRecuValidation'));
        
        $return = $this->_listValidants();
        $nbUserValideur = count($return['validants']);
        $nbUserConsultant = count($return['consultants']);
        
        $queryNormes = [
            'conditions' => [
                'abroger' => false
            ],
            'fields' => ['id', 'norme', 'numero', 'libelle', 'description'],
            'order' => ['norme', 'numero']
        ];
        $normes = $this->Norme->find('all', $queryNormes);
        
        $options_normes = Hash::combine(
            $normes,
            '{n}.Norme.id',
            [
                '%s-%s : %s',
                '{n}.Norme.norme',
                '{n}.Norme.numero',
                '{n}.Norme.libelle'
            ]
        );
        $this->set(compact('options_normes'));
        
        // Traitement reçu pour validation
        $query = $this->Banettes->queryRecuValidation();
        $banettes['recuValidation'] = [
            'results' => $this->Fiche->find('all', $query + ['limit' => 0]),
            'count' => $this->Fiche->find('count', $query),
            'nbUserValideur' => $nbUserValideur,
            'nbUserConsultant' => $nbUserConsultant,
            'enumsNormes' => $this->_enumsNorme()
        ];
        $this->set('banettes', $banettes);
        
        $this->set('validants', $return['validants']);
        $this->set('consultants', $return['consultants']);
    }

    /**
     * Fonction qui récupère tous les traitements reçus pour consultation
     * 
     * @access public
     * @created 13/02/2017
     * @version V1.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    public function recuConsultation() {
        $this->set('title', __d('pannel', 'pannel.titreTraitementConsultation'));

        // Traitement reçu pour consultation
        $query = $this->Banettes->queryRecuConsultation();
        $banettes['recuConsultation'] = [
            'results' => $this->Fiche->find('all', $query + ['limit' => 0]),
            'count' => $this->Fiche->find('count', $query),
            'enumsNormes' => $this->_enumsNorme()
        ];
        $this->set('banettes', $banettes);
    }

    /**
     * Requète récupérant les fiches validées par le DPO
     * 
     * @access public
     * @created 02/12/2015
     * @version V1.0.0
     */
    public function archives() {
        $this->set('title', __d('pannel', 'pannel.titreTraitementValidee'));

        // Mes traitements validés et insérés au registre
        $query = $this->Banettes->queryArchives();
        $banettes['archives'] = [
            'results' => $this->Fiche->find('all', $query + ['limit' => 0]),
            'count' => $this->Fiche->find('count', $query),
            'enumsNormes' => $this->_enumsNorme()
        ];
        $this->set('banettes', $banettes);
    }

    /**
     * Fonction appelée pour le composant parcours, permettant d'afficher le parcours parcouru par une fiche et les commentaires liés (uniquement ceux visibles par l'utilisateur)
     * 
     * @param int $id
     * @return type
     * 
     * @access public
     * @created 02/12/2015
     * @version V1.0.0
     */
    public function parcours($id) {
        $this->Droits->assertRecordAuthorized('Fiche', $id);

        $parcours = $this->EtatFiche->find('all', [
            'conditions' => [
                'EtatFiche.fiche_id' => $id,
            ],
            'contain' => [
                'Modification' => [
                    'id',
                    'modif',
                    'created'
                ],
                'Fiche' => [
                    'id',
                    'organisation_id',
                    'user_id',
                    'created',
                    'modified',
                    'User' => [
                        'id',
                        'nom',
                        'prenom',
                        'nom_complet'
                    ],
                ],
                'User' => [
                    'id',
                    'nom_complet'
                ],
                'Commentaire' => [
                    'User' => [
                        'id',
                        'nom',
                        'prenom',
                        'nom_complet'
                    ]
                ],
            ],
            'order' => [
                'EtatFiche.id DESC'
            ]
        ]);

        return $parcours;
    }

    /**
     * Fonction permettant d'afficher tout les traitements passer par le DPO 
     * ou le valideur ou l'administrateur
     * 
     * @access public
     * @created 10/05/2016
     * @version V1.0.0
     */
    public function consulte() {
        $this->set('title', __d('pannel', 'pannel.titreTraitementVu'));

        $return = $this->_listValidants();
        $nbUserValideur = count($return['validants']);
        
        // Etat des traitements passés en ma possession
        $query = $this->Banettes->queryConsulte();
        $banettes['consulte'] = [
            'results' => $this->Fiche->find('all', $query + ['limit' => 0]),
            'count' => $this->Fiche->find('count', $query),
            'nbUserValideur' => $nbUserValideur,
            'enumsNormes' => $this->_enumsNorme()
        ];
        $this->set('banettes', $banettes);

        $this->set('validants', $return['validants']);
    }

    /**
     * @param int $id
     * @return type
     * 
     * @access public
     * @created 02/12/2015
     * @version V1.0.0
     */
    public function getHistorique($id) {
        $this->Droits->assertRecordAuthorized('Fiche', $id);

        $historique = $this->Historique->find('all', [
            'conditions' => ['fiche_id' => $id],
            'order' => [
                'created DESC',
                'id DESC'
            ]
        ]);

        return $historique;
    }

    /**
     * Fonction de suppression de toute les notifications d'un utilisateur
     * 
     * @access public
     * @created 02/12/2015
     * @version V1.0.0
     */
    public function dropNotif() {
        $success = true;
        $this->Notification->begin();

        $success = $success && $this->Notification->deleteAll([
                    'Notification.user_id' => $this->Auth->user('id'),
                    false
        ]);

        if ($success == true) {
            $this->Notification->commit();
        } else {
            $this->Notification->rollback();
        }

        $this->redirect($this->referer());
    }

    /**
     * Fonction de suppression d'une notification d'un utilisateur
     * 
     * @access public
     * @created 20/01/2016
     * @version V1.0.0
     */
    public function supprimerLaNotif($idFiche) {
        $success = true;
        $this->Notification->begin();

        $success = $success && $this->Notification->deleteAll([
                    'Notification.fiche_id' => $idFiche,
                    'Notification.user_id' => $this->Auth->user('id')
        ]);

        if ($success == true) {
            $this->Notification->commit();
        } else {
            $this->Notification->rollback();
        }
    }

    /**
     * Permet de mettre dans la base de donner les notifications deja afficher 
     * quand on fermer la pop-up avec le bouton FERMER
     * 
     * @access public
     * @created 02/12/2015
     * @version V1.0.0
     */
    public function validNotif() {
        $success = true;
        $this->Notification->begin();

        $success = $success && $this->Notification->updateAll([
                    'Notification.afficher' => true
                        ], [
                    'Notification.user_id' => $this->Auth->user('id')
                ]) !== false;

        if ($success == true) {
            $this->Notification->commit();
        } else {
            $this->Notification->rollback();
        }

        $this->redirect($this->referer());
    }

    /**
     * Permet de mettre en base les notifs deja afficher
     * 
     * @param int $idFicheEnCourAffigage
     * 
     * @access public
     *
     * @created 20/01/2016
     * @version V1.0.0
     *
     * @modified 18/03/2021
     * @version V2.0.3
     */
    public function notifAfficher($idFicheEnCourAffigage)
    {
        $success = true;
        $this->Notification->begin();

        $success = $success && $this->Notification->updateAll([
            'Notification.afficher' => true
                ], [
            'Notification.user_id' => $this->Auth->user('id'),
            'Notification.fiche_id' => $idFicheEnCourAffigage
        ]) !== false;

        if ($success == true) {
            $this->Notification->commit();
        } else {
            $this->Notification->rollback();
        }
    }

    /**
     * @return type
     * 
     * @access protected
     * @created 02/12/2015
     * @version V1.0.0
     */
    protected function _listValidants() {
        // Requète récupérant les utilisateurs ayant le droit de consultation
        $queryConsultants = [
            'fields' => [
                'User.id',
                'User.nom',
                'User.prenom'
            ],
            'joins' => [
                $this->Droit->join('OrganisationUser', ['type' => "INNER"]),
                $this->Droit->OrganisationUser->join('User', ['type' => "INNER"])
            ],
            'recursive' => -1,
            'conditions' => [
                'OrganisationUser.organisation_id' => $this->Session->read('Organisation.id'),
                'User.id != ' . $this->Auth->user('id'),
                'Droit.liste_droit_id' => ListeDroit::VISER_TRAITEMENT
            ],
        ];
        $consultants = $this->Droit->find('all', $queryConsultants);
        $consultants = Hash::combine($consultants, '{n}.User.id', [
                    '%s %s',
                    '{n}.User.prenom',
                    '{n}.User.nom'
        ]);
        $return = ['consultants' => $consultants];


        // Requète récupérant les utilisateurs ayant le droit de validation
        if ($this->Session->read('Organisation.dpo') != null) {
            $dpo = $this->Session->read('Organisation.dpo');
        } else {
            $dpo = 0;
        }

        $queryValidants = [
            'fields' => [
                'User.id',
                'User.nom',
                'User.prenom'
            ],
            'joins' => [
                $this->Droit->join('OrganisationUser', ['type' => "INNER"]),
                $this->Droit->OrganisationUser->join('User', ['type' => "INNER"])
            ],
            'conditions' => [
                'OrganisationUser.organisation_id' => $this->Session->read('Organisation.id'),
                'NOT' => [
                    'User.id' => [
                        $this->Auth->user('id'),
                        $dpo
                    ]
                ],
                'Droit.liste_droit_id' => ListeDroit::VALIDER_TRAITEMENT
            ]
        ];
        $validants = $this->Droit->find('all', $queryValidants);
        $validants = Hash::combine($validants, '{n}.User.id', [
                    '%s %s',
                    '{n}.User.prenom',
                    '{n}.User.nom'
        ]);
        $return['validants'] = $validants;

        return $return;
    }

    /**
     * 
     * @param type $id
     * 
     * @access protected
     * @created 07/03/2017
     * @version V1.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    protected function _typeDeclarationRemplie($id) {
        $typeDeclaration = $this->Valeur->find('first', [
            'conditions' => [
                'fiche_id' => $id,
                'champ_name' => 'typedeclaration'
            ]
        ]);

        if (!empty($typeDeclaration)) {
            if ($typeDeclaration['Valeur']['valeur'] != ' ') {
                $remplie = 'true';
            } else {
                $remplie = 'false';
            }
        } else {
            $remplie = 'false';
        }

        return($remplie);
    }
    
    protected function _enumsNorme() {
        return ($this->Norme->enums());
    }
    
    public function checkEmail(){
        App::uses('CakeEmail', 'Network/Email');
        
        $Email = new CakeEmail('email');
        //$Email->from('noreply@adullact.org');
        $Email->subject("email test web-DPO");
        $Email->to("theo.guillon@libriciel.coop");

        $result = null;

        debug("envoie de l'e-mail");
        try {
            $result = $Email->send("contenu du mail de test");
        } catch (Exception $e) {
            debug($e->getMessage());
        }

        debug($result);
        $this->autoRender = false;
    }

    /**
     * Fonction qui récupère tous les traitements en cours de rédaction
     *
     * @access public
     * @created 10/12/2019
     * @version V2.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    public function initialisation() {
        $this->set('title', __d('pannel', 'pannel.titreInitialisationTraitement'));

        $redacteurs = $this->_listRedacteurs();

        // Initialisation d'un traitement
        $query = $this->Banettes->queryInitialisationTraitement();
        $banettes['initialisation'] = [
            'results' => $this->Fiche->find('all', $query + ['limit' => 0]),
            'count' => $this->Fiche->find('count', $query),
            'nbUserValideur' => 0,
            'nbUserConsultant' => 0,
            'enumsNormes' => $this->_enumsNorme()
        ];

        $this->set(compact('banettes', 'redacteurs'));
    }

    /**
     * Liste des rédacteurs de l'entité
     *
     * @return type
     *
     * @access protected
     * @created 11/12/2019
     * @version V2.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    protected function _listRedacteurs() {
        // Requète récupérant les utilisateurs ayant le droit de rédaction
        $queryRedacteurs = [
            'fields' => [
                'User.id',
                'User.nom',
                'User.prenom',
                $this->User->vfNomComplet()
            ],
            'joins' => [
                $this->Droit->join('OrganisationUser', ['type' => "INNER"]),
                $this->Droit->OrganisationUser->join('User', ['type' => "INNER"])
            ],
            'recursive' => -1,
            'conditions' => [
                'OrganisationUser.organisation_id' => $this->Session->read('Organisation.id'),
                'User.id != ' . $this->Auth->user('id'),
                'Droit.liste_droit_id' => ListeDroit::REDIGER_TRAITEMENT
            ],
        ];
        $droitsRedacteurs = $this->Droit->find('all', $queryRedacteurs);
        $redacteurs = Hash::combine($droitsRedacteurs, '{n}.User.id', [
            '%s %s',
            '{n}.User.prenom',
            '{n}.User.nom'
        ]);

        return $redacteurs;
    }

    /**
     * Visualisation de tous les traitements de l'entité
     *
     * @return type
     *
     * @access public
     * @created 12/12/2019
     * @version V2.0.0
     * @author Théo GUILLON <theo.guillon@libriciel.coop>
     */
    public function all_traitements() {
        $this->set('title', __d('pannel', 'pannel.titreAllTraitement'));

        // Tous les traitements
        $query = $this->Banettes->queryAllTraitements();

        if ($this->request->is('post')) {
            $conditions = [];

            // Filtre sur l'utilisateur à l'origine du traitement
            if (!empty($this->request->data['Filtre']['filtreEtatTraitement'])) {
                $conditions['EtatFiche.etat_id'] = $this->request->data['Filtre']['filtreEtatTraitement'];
            }

            $query['conditions'] += $conditions;
        }

        $options = [
            'etatTraitement' => [
                EtatFiche::ENCOURS_REDACTION => __d('pannel', 'pannel.textEtatEnCoursRedaction'),
                EtatFiche::ENCOURS_VALIDATION => __d('pannel', 'pannel.textEtatEnAttente'),
                EtatFiche::REFUSER => __d('pannel', 'pannel.textEtatRefuser'),
                EtatFiche::DEMANDE_AVIS => __d('pannel', 'pannel.textEtatDemandeAvis'),
                EtatFiche::REPLACER_REDACTION => __d('pannel', 'pannel.textEtatReplacerRedaction'),
                EtatFiche::REPONSE_AVIS => __d('pannel', 'pannel.textEtatConsulte'),
                EtatFiche::INITIALISATION_TRAITEMENT => __d('pannel', 'pannel.textEtatInitialisationTraitement'),
                EtatFiche::TRAITEMENT_INITIALISE_RECU => __d('pannel', 'pannel.textEtatTraitementInitialiseEnAttenteRedaction')
            ]
        ];

        $banettes['allTraitements'] = [
            'results' => $this->Fiche->find('all', $query + ['limit' => 0]),
            'count' => $this->Fiche->find('count', $query),
            'enumsNormes' => $this->_enumsNorme(),
        ];

        $this->set(compact('banettes', 'options'));
    }
}
